import 'package:flutter/material.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:senior_design_tokens/senior_design_tokens.dart';

class SeniorTextField extends StatefulWidget {
  final String? label;
  final IconData? leftIcon;
  final IconData? rightIcon;
  final String? helper;
  final String? hintText;
  final bool disabled;
  final String? initialValue;
  final FormFieldValidator<String>? validator;
  final int? maxLength;
  final int? minLines;
  final int? maxLines;
  final bool autocorrect;
  final bool readOnly;
  final Function(String)? onChanged;
  final Function()? onTap;
  final Function()? onEditingComplete;
  final bool autofocus;
  final TextEditingController? controller;
  final TextInputType? keyboardType;
  final bool obscureText;

  const SeniorTextField({
    Key? key,
    this.label,
    this.leftIcon,
    this.rightIcon,
    this.helper,
    this.hintText,
    this.disabled = false,
    this.initialValue,
    this.validator,
    this.maxLength,
    this.minLines,
    this.maxLines = 1,
    this.autocorrect = true,
    this.readOnly = false,
    this.onChanged,
    this.onTap,
    this.onEditingComplete,
    this.autofocus = false,
    this.controller,
    this.keyboardType,
    this.obscureText = false,
  }) : super(key: key);

  @override
  _SeniorTextField createState() => _SeniorTextField();
}

class _SeniorTextField extends State<SeniorTextField> {
  late TextEditingController _controller;
  late FocusNode _focusNode;
  String? _validMsg;
  bool _hasFocus = false;
  bool _hasError = false;
  String? _errorMsg;


  @override
  void initState() {
    super.initState();
    this._focusNode = FocusNode();
    this._focusNode.addListener(_onFocusChange);

    this._controller = widget.controller ?? TextEditingController();

    this._internalValidate(value: widget.initialValue, validateAsError: false);
  }

  @override
  void dispose() {
    this._focusNode.dispose();
    super.dispose();
  }

  void _onFocusChange() {
    setState(() {
      this._hasFocus = this._focusNode.hasFocus;
    });

    if (!this._hasFocus) {
      this._internalValidate(value: this._controller.text);
    }
  }

  void _internalValidate({String? value, bool validateAsError = true}) {
    if (widget.validator != null) {
      setState(() {
        this._validMsg = widget.validator!(value);

        if(validateAsError) {
          this._hasError = this._validMsg != null;
          this._errorMsg = this._validMsg;
        }
      });
    }
  }

  String? _validate(String? value) {
    this._internalValidate(value: value);
    return this._validMsg;
  }

  void _onChanged(String value) {
    this._internalValidate(value: value);

    if (widget.onChanged != null) {
      return widget.onChanged!(value);
    }
  }

  Widget? _buildFaIcon(IconData? icon) {
    if (icon != null) {
      return Padding(
        padding: const EdgeInsets.symmetric(
          horizontal: 12.0,
          vertical: 16.0,
        ),
        child: FaIcon(
          icon,
          size: SeniorTypographyIconSize.CTA,
          color: SeniorColors.neutralColor900,
        ),
      );
    }
    return null;
  }

  Widget _buildTextField({required Brightness theme}) {
    return Container(
      decoration: BoxDecoration(
        borderRadius: BorderRadius.only(
          topLeft: Radius.circular(SeniorRadius.xbig),
          topRight: Radius.circular(SeniorRadius.xbig),
          bottomRight: Radius.circular(SeniorRadius.xsmall),
          bottomLeft: Radius.circular(SeniorRadius.xsmall),
        ),
        border: Border.all(
          color: _hasFocus
              ? _hasError
                  ? SeniorColors.manchesterColorRed
                  : SeniorColors.primaryColor
              : Colors.transparent,
          width: 2.0,
        ),
      ),
      child: TextFormField(
        key: widget.key,
        style: SeniorTypography.text(color: SeniorColors.neutralColor900),
        onChanged: this._onChanged,
        onTap: widget.onTap,
        validator: this._validate,
        focusNode: this._focusNode,
        maxLength: widget.maxLength,
        minLines: widget.minLines,
        maxLines: widget.maxLines,
        autocorrect: widget.autocorrect,
        readOnly: widget.readOnly,
        onEditingComplete: widget.onEditingComplete,
        autofocus: widget.autofocus,
        enabled: !widget.disabled,
        controller: this._controller,
        keyboardType: widget.keyboardType,
        obscureText: widget.obscureText,
        decoration: InputDecoration(
          labelText: widget.label,
          labelStyle: this._hasFocus
              ? SeniorTypography.title(
                  color: this._hasError
                      ? SeniorColors.manchesterColorRed
                      : SeniorColors.primaryColor500,
                )
              : SeniorTypography.text(
                  color: this._hasError
                      ? SeniorColors.manchesterColorRed
                      : SeniorColors.neutralColor900,
                ),
          prefixIcon: this._buildFaIcon(widget.leftIcon),
          suffixIcon: this._buildFaIcon(widget.rightIcon),
          filled: true,
          fillColor: SeniorColors.neutralColor100,
          focusedBorder: InputBorder.none,
          errorStyle: TextStyle(
            fontSize: 0,
            height: 0,
          ),
          focusedErrorBorder: InputBorder.none,
          hintText: widget.hintText,
          hintStyle: SeniorTypography.text(color: SeniorColors.neutralColor600),
        ),
      ),
    );
  }

  Widget _buildHelper({required Brightness theme}) {
    return Padding(
      padding: const EdgeInsets.symmetric(horizontal: 14.0, vertical: 4.0),
      child: Text(
        this._errorMsg ?? widget.helper ?? '',
        style: SeniorTypography.helper(
          color: this._hasError
              ? SeniorColors.manchesterColorRed
              : SeniorColors.neutralColor700,
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    Brightness theme = MediaQuery.of(context).platformBrightness;

    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        this._buildTextField(theme: theme),
        this._buildHelper(theme: theme),
      ],
    );
  }
}
